from typing import List, Literal, Optional
from . import AsyncMostClient
from .types import Audio
from .search_types import SearchParams


class AsyncMostSearcher(object):
    def __init__(self, client: AsyncMostClient,
                 data_source: Literal["text", "audio"]):
        self.client = client
        self.data_source = data_source

    async def count(self,
                    filter: Optional[SearchParams] = None):
        if filter is None:
            filter = SearchParams()

        resp = await self.client.get(f"/{self.client.client_id}/{self.data_source}/count",
                                     params={
                                         "filter": filter.to_json(),
                                     })
        if resp.status_code >= 400:
            raise RuntimeError("Audio can't be indexed")
        return resp.json()


    async def distinct(self,
                       key: str,
                       filter: Optional[SearchParams] = None) -> List[str]:
        """
        Distinct values of key.
        :param key: key should be stored in info (fetch_info, store_info)
        :return:
        """
        if filter is None:
            filter = SearchParams()
        resp = await self.client.get(f"/{self.client.client_id}/{self.data_source}/distinct",
                               params={"filter": filter.to_json(),
                                       "key": key})
        if resp.status_code >= 400:
            raise RuntimeError("Key is not valid")
        return resp.json()

    async def search(self,
                     filter: Optional[SearchParams] = None,
                     limit: int = 10) -> List[Audio]:
        if filter is None:
            filter = SearchParams()
        resp = await self.client.get(f"/{self.client.client_id}/{self.data_source}/search",
                                     params={
                                         "filter": filter.to_json(),
                                         "limit": limit,
                                     })
        if resp.status_code >= 400:
            raise RuntimeError("Audio can't be indexed")
        audio_list = resp.json()
        return self.client.retort.load(audio_list, List[Audio])



import re
from dataclasses import dataclass
from datetime import datetime
from typing import Dict, List, Literal, Optional, Union
from dataclasses_json import DataClassJsonMixin, dataclass_json


@dataclass_json
@dataclass
class StoredAudioData(DataClassJsonMixin):
    id: str
    data: Dict[str, Union[str, int, float]]


@dataclass_json
@dataclass
class Audio(DataClassJsonMixin):
    id: str
    url: str


@dataclass_json
@dataclass
class Text(DataClassJsonMixin):
    id: str


@dataclass_json
@dataclass
class SubcolumnResult(DataClassJsonMixin):
    name: str
    score: Optional[int] = None
    description: str = ""


@dataclass_json
@dataclass
class ColumnResult(DataClassJsonMixin):
    name: str
    subcolumns: List[SubcolumnResult]


@dataclass_json
@dataclass
class Column(DataClassJsonMixin):
    name: str
    subcolumns: List[str]


@dataclass_json
@dataclass
class Script(DataClassJsonMixin):
    columns: List[Column]


@dataclass_json
@dataclass
class ScriptScoreMapping(DataClassJsonMixin):
    column: str
    subcolumn: str
    from_score: int
    to_score: int


@dataclass_json
@dataclass
class JobStatus(DataClassJsonMixin):
    status: Literal["not_found", "pending", "completed", "error"]


@dataclass_json
@dataclass
class Result(DataClassJsonMixin):
    id: str
    text: Optional[str] = None
    url: Optional[str] = None
    results: Optional[List[ColumnResult]] = None
    created_at: Optional[datetime] = None
    applied_at: Optional[datetime] = None

    def get_script(self) -> Optional[Script]:
        if self.results is None:
            return None

        return Script(columns=[Column(name=column_result.name,
                                      subcolumns=[subcolumn_result.name
                                                  for subcolumn_result in column_result.subcolumns])
                               for column_result in self.results])


@dataclass_json
@dataclass
class DialogSegment(DataClassJsonMixin):
    start_time_ms: int
    end_time_ms: int
    text: str
    speaker: str
    emotion: Optional[str] = None
    intensity: Optional[float] = None

    def to_text(self):
        if self.emotion is None:
            return f'{self.speaker}: {self.text}\n'
        else:
            return f'{self.speaker}: <emotion name="{self.emotion}" intensity="{self.intensity}">{self.text}</emotion>\n'


@dataclass_json
@dataclass
class Dialog(DataClassJsonMixin):
    segments: List[DialogSegment]

    def to_text(self):
        return ''.join([segment.to_text()
                        for segment in self.segments])


@dataclass_json
@dataclass
class DialogResult(DataClassJsonMixin):
    id: str
    dialog: Optional[Dialog] = None
    url: Optional[str] = None
    results: Optional[List[ColumnResult]] = None


@dataclass_json
@dataclass
class HumanFeedback(DataClassJsonMixin):
    data_point_id: str
    data_point_type: Literal["audio", "text"]
    column_name: str
    subcolumn_name: str
    score: int
    description: str = ""

    @classmethod
    def calculate_accuracy(cls,
                           preds: List["HumanFeedback"],
                           gt: List["HumanFeedback"]) -> float:
        preds = {(y_pred.data_point_id, y_pred.column_name, y_pred.subcolumn_name): y_pred.score
                 for y_pred in preds}
        gt = {(y_true.data_point_id, y_true.column_name, y_true.subcolumn_name): y_true.score
              for y_true in gt}
        common_keys = set(preds.keys()) & set(gt.keys())
        return sum((preds[key] == gt[key]) for key in common_keys) / len(common_keys)


@dataclass_json
@dataclass
class Usage(DataClassJsonMixin):
    apply_audio_async: int
    apply_audio_async_duration: int
    apply_text_async: int

    apply_audio: int
    apply_audio_duration: int
    apply_text: int

    upload_audio: int
    upload_audio_duration: int
    upload_text: int

    start_dt: datetime
    end_dt: datetime


def is_valid_objectid(oid: str) -> bool:
    """
    Check if a given string is a valid MongoDB ObjectId (24-character hex).
    """
    return bool(re.fullmatch(r"^[0-9a-fA-F]{24}$", oid))


def is_valid_id(smth_id: Optional[str]) -> bool:
    if smth_id is None:
        return False

    if smth_id.startswith("most-"):
        smth_id = smth_id[5:]

    return is_valid_objectid(smth_id)



import io
import os
import uuid
from datetime import datetime, timezone
from pathlib import Path
from typing import Dict, List, Optional, Union
import httpx
import json5
from adaptix import Retort, loader
from pydub import AudioSegment
from most._constrants import DEFAULT_MAX_RETRIES
from most.score_calculation import ScoreCalculation
from most.types import (
    Audio,
    DialogResult,
    JobStatus,
    Result,
    Script,
    StoredAudioData,
    Text,
    is_valid_id, ScriptScoreMapping, Dialog, Usage,
)


class AsyncMostClient(object):
    retort = Retort(recipe=[
        loader(int, lambda x: x if isinstance(x, int) else int(x)),
        loader(float, lambda x: x if isinstance(x, float) else float(x)),
        loader(datetime, lambda x: datetime.fromtimestamp(x).astimezone(tz=timezone.utc) if isinstance(x, (int, float)) else datetime.fromisoformat(x)),
    ])

    def __init__(self,
                 client_id=None,
                 client_secret=None,
                 model_id=None,

                 base_url: str | httpx.URL | None = None,
                 timeout: Union[float, httpx.Timeout] = 1e10,
                 max_retries: int = DEFAULT_MAX_RETRIES,
                 # retry_delay: float = 0,
                 http_client: httpx.AsyncClient | None = None):
        super(AsyncMostClient, self).__init__()
        self.client_id = client_id
        self.client_secret = client_secret

        if self.client_id is None:
            credentials = self.load_credentials()
            self.client_id = credentials.get('client_id')
            self.client_secret = credentials.get('client_secret')

            if self.client_id is None:
                print("Visit: https://app.the-most.ai/integrations and get clientId, clientSecret")
                self.client_id = input("Please enter your client ID: ")
                self.client_secret = input("Please enter your client secret: ")
                self.save_credentials()
        else:
            self.save_credentials()

        if base_url is None:
            base_url = os.environ.get("MOST_BASE_URL")
        if base_url is None:
            base_url = f"https://api.the-most.ai/api/external"

        if http_client is None:
            http_client = httpx.AsyncClient(base_url=base_url,
                                            timeout=timeout,
                                            follow_redirects=True,
                                            transport=httpx.AsyncHTTPTransport(retries=max_retries))
        # self.max_retries = max_retries
        # self.retry_delay = retry_delay
        self.session = http_client
        self.access_token = None
        self.model_id = model_id
        self.score_modifier: Optional[ScoreCalculation] = None

    async def __aenter__(self):
        await self.session.__aenter__()
        await self.refresh_access_token()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.session.__aexit__(exc_type, exc_val, exc_tb)

    @property
    def cache_path(self):
        path = Path.home() / ".most"
        path.mkdir(parents=True, exist_ok=True)
        return path

    def load_credentials(self):
        path = self.cache_path / "credentials.json"
        if not path.exists():
            return {}
        else:
            return json5.loads(path.read_text())

    def save_credentials(self):
        path = self.cache_path / "credentials.json"
        path.write_text(json5.dumps({
            "client_id": self.client_id,
            "client_secret": self.client_secret,
        }))

    def clone(self):
        client = AsyncMostClient(client_id=self.client_id,
                                 client_secret=self.client_secret,
                                 model_id=self.model_id)
        client.access_token = self.access_token
        client.session = self.session
        client.score_modifier = self.score_modifier
        return client

    def with_model(self, model_id):
        client = self.clone()
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")
        client.model_id = model_id
        client.score_modifier = None
        return client

    async def refresh_access_token(self):
        resp = await self.session.post("/access_token",
                                       json={"client_id": self.client_id,
                                             "client_secret": self.client_secret})
        access_token = resp.json()
        self.access_token = access_token

    async def get(self, url, **kwargs):
        if self.access_token is None:
            await self.refresh_access_token()
        headers = kwargs.pop("headers", {})
        headers.update({"Authorization": "Bearer %s" % self.access_token})
        resp = await self.session.get(url,
                                      headers=headers,
                                      timeout=None,
                                      **kwargs)
        if resp.status_code == 401:
            await self.refresh_access_token()
            return await self.get(url,
                                  headers=headers,
                                  **kwargs)

        if resp.status_code >= 400:
            raise RuntimeError(resp.json()['message'] if resp.headers.get("Content-Type") == "application/json" else "Something went wrong.")
        resp.raise_for_status()
        return resp

    async def put(self, url, **kwargs):
        if self.access_token is None:
            await self.refresh_access_token()
        headers = kwargs.pop("headers", {})
        headers.update({"Authorization": "Bearer %s" % self.access_token})
        resp = await self.session.put(url,
                                      headers=headers,
                                      timeout=None,
                                      **kwargs)
        if resp.status_code == 401:
            await self.refresh_access_token()
            return await self.put(url,
                                  headers=headers,
                                  **kwargs)
        if resp.status_code >= 400:
            raise RuntimeError(resp.json()['message'] if resp.headers.get("Content-Type") == "application/json" else "Something went wrong.")
        return resp

    async def post(self, url,
                   data=None,
                   json=None,
                   **kwargs):
        if self.access_token is None:
            await self.refresh_access_token()
        headers = kwargs.pop("headers", {})
        headers.update({"Authorization": "Bearer %s" % self.access_token})
        resp = await self.session.post(url,
                                       data=data,
                                       json=json,
                                       headers=headers,
                                       timeout=None,
                                       **kwargs)
        if resp.status_code == 401:
            await self.refresh_access_token()
            return await self.post(url,
                                   data=data,
                                   json=json,
                                   headers=headers,
                                   **kwargs)
        if resp.status_code >= 400:
            raise RuntimeError(resp.json()['message'] if resp.headers.get(
                "Content-Type") == "application/json" else "Something went wrong.")
        return resp

    async def upload_audio(self, audio_path) -> Audio:
        with open(audio_path, mode='rb') as f:
            resp = await self.post(f"/{self.client_id}/upload",
                                   files={"audio_file": f})
        return self.retort.load(resp.json(), Audio)

    async def upload_audio_segment(self, audio: AudioSegment,
                                   audio_name: Optional[str] = None) -> Audio:
        f = io.BytesIO()
        audio.export(f, format="mp3")
        f.seek(0)
        if audio_name is None:
            audio_name = uuid.uuid4().hex + ".mp3"
        resp = await self.post(f"/{self.client_id}/upload",
                               files={"audio_file": (audio_name, f, 'audio/mp3')})
        return self.retort.load(resp.json(), Audio)

    async def upload_text(self, text: str) -> Text:
        resp = await self.post(f"/{self.client_id}/upload_text",
                               json={"text": text})
        return self.retort.load(resp.json(), Text)

    async def upload_dialog(self, dialog: Dialog) -> DialogResult:
        resp = await self.post(f"/{self.client_id}/upload_dialog",
                               json={"dialog": dialog})
        return self.retort.load(resp.json(), DialogResult)

    async def upload_audio_url(self, audio_url) -> Audio:
        resp = await self.post(f"/{self.client_id}/upload_url",
                               json={"audio_url": audio_url})
        return self.retort.load(resp.json(), Audio)

    async def list_audios(self,
                          offset: int = 0,
                          limit: int = 10,
                          query: Optional[Dict[str, str]] = None) -> List[Audio]:
        if query is None:
            query = {}
        resp = await self.get(f"/{self.client_id}/list?offset={offset}&limit={limit}",
                              params=query)
        audio_list = resp.json()
        return self.retort.load(audio_list, List[Audio])

    async def list_texts(self,
                         offset: int = 0,
                         limit: int = 10) -> List[Text]:
        resp = await self.get(f"/{self.client_id}/list_texts?offset={offset}&limit={limit}")
        texts_list = resp.json()
        return self.retort.load(texts_list, List[Text])

    async def get_model_script(self) -> Script:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        resp = await self.get(f"/{self.client_id}/model/{self.model_id}/script")
        return self.retort.load(resp.json(), Script)

    async def get_score_modifier(self):
        if self.score_modifier is None:
            if not is_valid_id(self.model_id):
                raise RuntimeError("Please choose valid model to apply. [try list_models()]")

            resp = await self.get(f"/{self.client_id}/model/{self.model_id}/score_mapping")
            score_mapping = self.retort.load(resp.json(), List[ScriptScoreMapping])
            self.score_modifier = ScoreCalculation(score_mapping)
        return self.score_modifier

    async def list_models(self):
        resp = await self.get("/list_models")
        return [self.with_model(model['model'])
                for model in resp.json()]

    async def apply(self, audio_id,
                    modify_scores: bool = False,
                    overwrite: bool = False) -> Result:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = await self.post(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/apply",
                               params={"overwrite": overwrite})
        result = self.retort.load(resp.json(), Result)
        if modify_scores:
            score_modifier = await self.get_score_modifier()
            result = score_modifier.modify(result)
        return result

    async def apply_on_text(self, text_id,
                            modify_scores: bool = False,
                            overwrite: bool = False) -> Result:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(text_id):
            raise RuntimeError("Please use valid text_id. [try text.id from list_texts()]")

        resp = await self.post(f"/{self.client_id}/text/{text_id}/model/{self.model_id}/apply",
                               params={"overwrite": overwrite})
        result = self.retort.load(resp.json(), Result)
        if modify_scores:
            score_modifier = await self.get_score_modifier()
            result = score_modifier.modify(result)
        return result

    async def transcribe_later(self, audio_id,
                               overwrite: bool = False) -> DialogResult:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = await self.post(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/transcribe_async",
                               params={"overwrite": overwrite})
        return self.retort.load(resp.json(), DialogResult)

    async def apply_later(self, audio_id,
                          modify_scores: bool = False,
                          overwrite: bool = False) -> Result:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = await self.post(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/apply_async",
                               params={"overwrite": overwrite})
        result = self.retort.load(resp.json(), Result)
        if modify_scores:
            score_modifier = await self.get_score_modifier()
            result = score_modifier.modify(result)
        return result

    async def apply_on_text_later(self, text_id,
                                  modify_scores: bool = False,
                                  overwrite: bool = False) -> Result:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(text_id):
            raise RuntimeError("Please use valid text_id. [try audio.id from list_texts()]")

        resp = await self.post(f"/{self.client_id}/text/{text_id}/model/{self.model_id}/apply_async",
                               params={"overwrite": overwrite})
        result = self.retort.load(resp.json(), Result)
        if modify_scores:
            score_modifier = await self.get_score_modifier()
            result = score_modifier.modify(result)
        return result

    async def get_job_status(self, audio_id) -> JobStatus:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = await self.post(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/apply_status")
        return self.retort.load(resp.json(), JobStatus)

    async def fetch_results(self, audio_id,
                            modify_scores: bool = False) -> Result:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = await self.get(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/results")
        result = self.retort.load(resp.json(), Result)
        if modify_scores:
            score_modifier = await self.get_score_modifier()
            result = score_modifier.modify(result)
        return result

    async def fetch_text(self, audio_id) -> Result:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = await self.get(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/text")
        return self.retort.load(resp.json(), Result)

    async def fetch_dialog(self, audio_id) -> DialogResult:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = await self.get(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/dialog")
        return self.retort.load(resp.json(), DialogResult)

    async def update_dialog(self, audio_id, dialog: Dialog) -> DialogResult:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = await self.put(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/dialog",
                              json={"dialog": dialog.to_dict()})
        return self.retort.load(resp.json(), DialogResult)

    async def export(self, audio_ids: List[str],
                     aggregated_by: Optional[str] = None,
                     aggregation_title: Optional[str] = None,
                     modify_scores: bool = False) -> str:
        if aggregation_title is None:
            aggregation_title = aggregated_by

        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        resp = await self.get(f"/{self.client_id}/model/{self.model_id}/export",
                              params={'audio_ids': ','.join(audio_ids),
                                      "aggregated_by": aggregated_by,
                                      "aggregation_title": aggregation_title,
                                      "modify_scores": modify_scores})
        return resp.next_request.url

    async def store_info(self,
                         audio_id: str,
                         data: Dict[str, Union[str, int, float]]) -> StoredAudioData:
        resp = await self.post(f"/{self.client_id}/audio/{audio_id}/info",
                               json={
                                   "data": data,
                               })
        return StoredAudioData.from_dict(resp.json())

    async def fetch_info(self, audio_id: str) -> StoredAudioData:
        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")
        resp = await self.get(f"/{self.client_id}/audio/{audio_id}/info")
        return StoredAudioData.from_dict(resp.json())

    async def __call__(self, audio_path: Path,
                       modify_scores: bool = False) -> Result:
        audio = await self.upload_audio(audio_path)
        return await self.apply(audio.id,
                                modify_scores=modify_scores)

    def __repr__(self):
        return "<AsyncMostClient(model_id='%s')>" % (self.model_id, )

    async def get_audio_segment_by_url(self, audio_url,
                                       format=None):
        if format is None:
            format = os.path.splitext(audio_url)[1]
            format = format.strip().lower()

        resp = await self.session.get(audio_url,
                                      timeout=None)
        if resp.status_code >= 400:
            raise RuntimeError("Audio url is not accessable")

        audio = AudioSegment.from_file(io.BytesIO(resp.content),
                                       format=format)
        return audio

    async def index_audio(self, audio_id: str) -> None:
        resp = await self.post(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/indexing")
        if resp.status_code >= 400:
            raise RuntimeError("Audio can't be indexed")
        return None

    async def get_usage(self,
                        start_dt: datetime,
                        end_dt: datetime):
        resp = await self.get(f"/{self.client_id}/model/{self.model_id}/usage",
                              params={'start_dt': start_dt.astimezone(timezone.utc).isoformat(),
                                      'end_dt': end_dt.astimezone(timezone.utc).isoformat()})
        resp.raise_for_status()
        return self.retort.load(resp.json(), Usage)



from typing import Dict, Tuple, List, Optional, Literal
from dataclasses_json import dataclass_json, DataClassJsonMixin
from dataclasses import dataclass, replace
from .types import Result, ScriptScoreMapping


@dataclass_json
@dataclass
class ScoreCalculation(DataClassJsonMixin):
    score_mapping: List[ScriptScoreMapping]

    def modify(self, result: Optional[Result]):
        score_mapping = {
            (sm.column, sm.subcolumn, sm.from_score): sm.to_score
            for sm in self.score_mapping
        }
        if result is None:
            return None
        result = replace(result)
        for column_result in result.results:
            for subcolumn_result in column_result.subcolumns:
                subcolumn_result.score = score_mapping.get((column_result.name,
                                                            subcolumn_result.name,
                                                            subcolumn_result.score),
                                                           subcolumn_result.score)

        return result

    def unmodify(self, result: Optional[Result]):
        score_mapping = {
            (sm.column, sm.subcolumn, sm.to_score): sm.from_score
            for sm in self.score_mapping
        }
        if result is None:
            return None
        result = replace(result)
        for column_result in result.results:
            for subcolumn_result in column_result.subcolumns:
                subcolumn_result.score = score_mapping.get((column_result.name,
                                                            subcolumn_result.name,
                                                            subcolumn_result.score),
                                                           subcolumn_result.score)

        return result

    def modify_single(self,
                      column: str, subcolumn: str,
                      from_score: int):
        for sm in self.score_mapping:
            if sm.column == column and sm.subcolumn == subcolumn and sm.from_score == from_score:
                return sm.to_score

    def unmodify_single(self,
                        column: str, subcolumn: str,
                        to_score: int,
                        bound: Literal["strict", "upper", "lower"] = "strict"):
        upper_from_score = None
        lower_from_score = None

        for sm in self.score_mapping:
            if sm.column == column and sm.subcolumn == subcolumn:
                if sm.to_score == to_score:
                    return sm.from_score

                if sm.to_score > to_score:
                    if upper_from_score is None or sm.to_score < upper_from_score[1]:
                        upper_from_score = (sm.from_score, sm.to_score)
                elif sm.to_score < to_score:
                    if lower_from_score is None or sm.to_score > lower_from_score[1]:
                        lower_from_score = (sm.from_score, sm.to_score)

        if bound == "strict":
            return None
        elif bound == "upper" and upper_from_score is not None:
            return upper_from_score[0]
        elif bound == "lower" and lower_from_score is not None:
            return lower_from_score[0]
        else:
            return None



import io
import os
import uuid
from datetime import datetime, timezone
from pathlib import Path
from typing import Dict, List, Optional, Union
import json5
import httpx
from adaptix import Retort, loader
from pydub import AudioSegment
from most._constrants import DEFAULT_MAX_RETRIES, DEFAULT_RETRY_DELAY
from most.score_calculation import ScoreCalculation
from most.types import (
    Audio,
    DialogResult,
    JobStatus,
    Result,
    Script,
    StoredAudioData,
    Text,
    is_valid_id, ScriptScoreMapping, Dialog, Usage,
)


class MostClient(object):
    retort = Retort(recipe=[
        loader(int, lambda x: x if isinstance(x, int) else int(x)),
        loader(float, lambda x: x if isinstance(x, float) else float(x)),
        loader(datetime, lambda x: datetime.fromtimestamp(x).astimezone(tz=timezone.utc) if isinstance(x, (int, float)) else datetime.fromisoformat(x)),
    ],)

    def __init__(self,
                 client_id=None,
                 client_secret=None,
                 model_id=None,

                 base_url: str | httpx.URL | None = None,
                 timeout: Union[float, httpx.Timeout] = 1e10,
                 max_retries: int = DEFAULT_MAX_RETRIES,
                 # retry_delay: float = DEFAULT_RETRY_DELAY,
                 http_client: httpx.Client | None = None):
        super(MostClient, self).__init__()
        self.client_id = client_id
        self.client_secret = client_secret

        if self.client_id is None:
            credentials = self.load_credentials()
            self.client_id = credentials.get('client_id')
            self.client_secret = credentials.get('client_secret')

            if self.client_id is None:
                print("Visit: https://app.the-most.ai/integrations and get clientId, clientSecret")
                self.client_id = input("Please enter your client ID: ")
                self.client_secret = input("Please enter your client secret: ")
                self.save_credentials()
        else:
            self.save_credentials()

        if base_url is None:
            base_url = os.environ.get("MOST_BASE_URL")
        if base_url is None:
            base_url = f"https://api.the-most.ai/api/external"

        if http_client is None:
            http_client = httpx.Client(base_url=base_url,
                                       timeout=timeout,
                                       follow_redirects=True,
                                       transport=httpx.HTTPTransport(retries=max_retries))
        # self.max_retries = max_retries
        # self.retry_delay = retry_delay
        self.session = http_client
        self.access_token = None
        self.model_id = model_id
        self.score_modifier = None

        self.refresh_access_token()

    @property
    def cache_path(self):
        path = Path.home() / ".most"
        path.mkdir(parents=True, exist_ok=True)
        return path

    def load_credentials(self):
        path = self.cache_path / "credentials.json"
        if not path.exists():
            return {}
        else:
            return json5.loads(path.read_text())

    def save_credentials(self):
        path = self.cache_path / "credentials.json"
        path.write_text(json5.dumps({
            "client_id": self.client_id,
            "client_secret": self.client_secret,
        }))

    def clone(self):
        client = MostClient(client_id=self.client_id,
                            client_secret=self.client_secret,
                            model_id=self.model_id)
        client.access_token = self.access_token
        client.session = self.session
        client.score_modifier = self.score_modifier
        return client

    def with_model(self, model_id):
        client = self.clone()
        client.model_id = model_id
        client.score_modifier = None
        return client

    def refresh_access_token(self):
        resp = self.session.post("/access_token",
                                 json={"client_id": self.client_id,
                                        "client_secret": self.client_secret},
                                 timeout=None)
        access_token = resp.json()
        self.access_token = access_token

    def get(self, url, **kwargs):
        if self.access_token is None:
            self.refresh_access_token()
        headers = kwargs.pop("headers", {})
        headers.update({"Authorization": "Bearer %s" % self.access_token})
        resp = self.session.get(url,
                                headers=headers,
                                timeout=None,
                                **kwargs)
        if resp.status_code == 401:
            self.refresh_access_token()
            return self.get(url,
                            headers=headers,
                            **kwargs)
        if resp.status_code >= 400:
            raise RuntimeError(resp.json()['message'] if resp.headers.get("Content-Type") == "application/json" else resp.content)
        return resp

    def put(self, url, **kwargs):
        if self.access_token is None:
            self.refresh_access_token()
        headers = kwargs.pop("headers", {})
        headers.update({"Authorization": "Bearer %s" % self.access_token})
        resp = self.session.put(url,
                                headers=headers,
                                timeout=None,
                                **kwargs)
        if resp.status_code == 401:
            self.refresh_access_token()
            return self.put(url,
                            headers=headers,
                            **kwargs)
        if resp.status_code >= 400:
            raise RuntimeError(resp.json()['message'] if resp.headers.get("Content-Type") == "application/json" else resp.content)
        return resp

    def post(self, url,
             data=None,
             json=None,
             **kwargs):
        if self.access_token is None:
            self.refresh_access_token()
        headers = kwargs.pop("headers", {})
        headers.update({"Authorization": "Bearer %s" % self.access_token})
        resp = self.session.post(url,
                                 data=data,
                                 json=json,
                                 headers=headers,
                                 timeout=None,
                                 **kwargs)
        if resp.status_code == 401:
            self.refresh_access_token()
            return self.post(url,
                             data=data,
                             json=json,
                             headers=headers,
                             **kwargs)
        if resp.status_code >= 400:
            raise RuntimeError(resp.json()['message'] if resp.headers.get("Content-Type") == "application/json" else "Something went wrong.")
        return resp

    def upload_text(self, text: str) -> Text:
        resp = self.post(f"/{self.client_id}/upload_text",
                         files={"text": text})
        return self.retort.load(resp.json(), Text)

    def upload_dialog(self, dialog: Dialog) -> DialogResult:
        resp = self.post(f"/{self.client_id}/upload_dialog",
                         json={"dialog": dialog})
        return self.retort.load(resp.json(), DialogResult)

    def upload_audio(self, audio_path) -> Audio:
        with open(audio_path, 'rb') as f:
            resp = self.post(f"/{self.client_id}/upload",
                             files={"audio_file": f})
        return self.retort.load(resp.json(), Audio)

    def upload_audio_segment(self, audio: AudioSegment,
                             audio_name: Optional[str] = None) -> Audio:
        f = io.BytesIO()
        audio.export(f, format="mp3")
        f.seek(0)
        if audio_name is None:
            audio_name = uuid.uuid4().hex + ".mp3"
        resp = self.post(f"/{self.client_id}/upload",
                         files={"audio_file": (audio_name, f, 'audio/mp3')})
        return self.retort.load(resp.json(), Audio)

    def upload_audio_url(self, audio_url) -> Audio:
        resp = self.post(f"/{self.client_id}/upload_url",
                         json={"audio_url": audio_url})
        return self.retort.load(resp.json(), Audio)

    def list_audios(self,
                    offset: int = 0,
                    limit: int = 10,
                    query: Optional[Dict[str, str]] = None) -> List[Audio]:
        if query is None:
            query = {}
        resp = self.get(f"/{self.client_id}/list?offset={offset}&limit={limit}",
                        params=query)
        audio_list = resp.json()
        return self.retort.load(audio_list, List[Audio])

    def list_texts(self,
                   offset: int = 0,
                   limit: int = 10) -> List[Text]:
        resp = self.get(f"/{self.client_id}/list_texts?offset={offset}&limit={limit}")
        texts_list = resp.json()
        return self.retort.load(texts_list, List[Text])

    def get_model_script(self) -> Script:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        resp = self.get(f"/{self.client_id}/model/{self.model_id}/script")
        return self.retort.load(resp.json(), Script)

    def get_score_modifier(self):
        if self.score_modifier is None:
            if not is_valid_id(self.model_id):
                raise RuntimeError("Please choose valid model to apply. [try list_models()]")

            resp = self.get(f"/{self.client_id}/model/{self.model_id}/score_mapping")
            score_mapping = self.retort.load(resp.json(), List[ScriptScoreMapping])
            self.score_modifier = ScoreCalculation(score_mapping)
        return self.score_modifier

    def list_models(self):
        resp = self.get("/list_models")
        return [self.with_model(model['model'])
                for model in resp.json()]

    def apply(self, audio_id,
              modify_scores: bool = False,
              overwrite: bool = False) -> Result:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = self.post(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/apply",
                         params={"overwrite": overwrite})
        result = self.retort.load(resp.json(), Result)
        if modify_scores:
            result = self.get_score_modifier().modify(result)
        return result

    def apply_on_text(self, text_id,
                      modify_scores: bool = False,
                      overwrite: bool = False) -> Result:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(text_id):
            raise RuntimeError("Please use valid text_id. [try text.id from list_texts()]")

        resp = self.post(f"/{self.client_id}/text/{text_id}/model/{self.model_id}/apply",
                         params={"overwrite": overwrite})
        result = self.retort.load(resp.json(), Result)
        if modify_scores:
            result = self.get_score_modifier().modify(result)
        return result

    def transcribe_later(self, audio_id,
                         overwrite: bool = False) -> DialogResult:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = self.post(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/transcribe_async",
                         params={"overwrite": overwrite})
        return self.retort.load(resp.json(), DialogResult)

    def apply_later(self, audio_id,
                    modify_scores: bool = False,
                    overwrite: bool = False) -> Result:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = self.post(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/apply_async",
                         params={"overwrite": overwrite})
        result = self.retort.load(resp.json(), Result)
        if modify_scores:
            result = self.get_score_modifier().modify(result)
        return result

    def apply_on_text_later(self, text_id,
                            modify_scores: bool = False,
                            overwrite: bool = False) -> Result:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(text_id):
            raise RuntimeError("Please use valid text_id. [try audio.id from list_texts()]")

        resp = self.post(f"/{self.client_id}/text/{text_id}/model/{self.model_id}/apply_async",
                         params={"overwrite": overwrite})
        result = self.retort.load(resp.json(), Result)
        if modify_scores:
            result = self.get_score_modifier().modify(result)
        return result

    def get_job_status(self, audio_id) -> JobStatus:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = self.post(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/apply_status")
        return self.retort.load(resp.json(), JobStatus)

    def fetch_results(self, audio_id,
                      modify_scores: bool = False) -> Result:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = self.get(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/results")
        result = self.retort.load(resp.json(), Result)
        if modify_scores:
            result = self.get_score_modifier().modify(result)
        return result

    def fetch_text(self, audio_id: str) -> Result:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = self.get(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/text")
        return self.retort.load(resp.json(), Result)

    def fetch_dialog(self, audio_id) -> DialogResult:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = self.get(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/dialog")
        return self.retort.load(resp.json(), DialogResult)

    def update_dialog(self, audio_id, dialog: Dialog) -> DialogResult:
        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = self.put(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/dialog",
                        json={"dialog": dialog.to_dict()})
        return self.retort.load(resp.json(), DialogResult)

    def export(self, audio_ids: List[str],
               aggregated_by: Optional[str] = None,
               aggregation_title: Optional[str] = None,
               modify_scores: bool = False) -> str:
        if aggregation_title is None:
            aggregation_title = aggregated_by

        if not is_valid_id(self.model_id):
            raise RuntimeError("Please choose valid model to apply. [try list_models()]")

        resp = self.get(f"/{self.client_id}/model/{self.model_id}/export",
                        params={'audio_ids': ','.join(audio_ids),
                                'aggregated_by': aggregated_by,
                                'aggregation_title': aggregation_title,
                                'modify_scores': modify_scores})
        return resp.url

    def store_info(self,
                   audio_id: str,
                   data: Dict[str, Union[str, int, float]]) -> StoredAudioData:
        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = self.post(f"/{self.client_id}/audio/{audio_id}/info",
                         json={
                             "data": data,
                         })
        return StoredAudioData.from_dict(resp.json())

    def fetch_info(self, audio_id: str) -> StoredAudioData:
        if not is_valid_id(audio_id):
            raise RuntimeError("Please use valid audio_id. [try audio.id from list_audios()]")

        resp = self.get(f"/{self.client_id}/audio/{audio_id}/info")
        return StoredAudioData.from_dict(resp.json())

    def __call__(self, audio_path: Path,
                 modify_scores: bool = False) -> Result:
        audio = self.upload_audio(audio_path)
        return self.apply(audio.id,
                          modify_scores=modify_scores)

    def __repr__(self):
        return "<MostClient(model_id='%s')>" % (self.model_id, )

    def get_audio_segment_by_url(self, audio_url,
                                 format=None):
        if format is None:
            format = os.path.splitext(audio_url)[1]
            format = format.strip().lower()

        resp = self.session.get(audio_url,
                                timeout=None)
        if resp.status_code >= 400:
            raise RuntimeError("Audio url is not accessable")

        audio = AudioSegment.from_file(io.BytesIO(resp.content),
                                       format=format)
        return audio

    def index_audio(self, audio_id: str) -> None:
        resp = self.post(f"/{self.client_id}/audio/{audio_id}/model/{self.model_id}/indexing")
        if resp.status_code >= 400:
            raise RuntimeError("Audio can't be indexed")
        return None

    def get_usage(self,
                  start_dt: datetime,
                  end_dt: datetime):
        resp = self.get(f"/{self.client_id}/model/{self.model_id}/usage",
                        params={'start_dt': start_dt.astimezone(timezone.utc).isoformat(),
                                'end_dt': end_dt.astimezone(timezone.utc).isoformat()})
        resp.raise_for_status()
        return self.retort.load(resp.json(), Usage)



import httpx


DEFAULT_TIMEOUT = httpx.Timeout(timeout=600.0, connect=5.0)
DEFAULT_MAX_RETRIES = 2
DEFAULT_RETRY_DELAY = 5
DEFAULT_CONNECTION_LIMITS = httpx.Limits(max_connections=1000, max_keepalive_connections=100)



from typing import List, Literal, Optional

from .api import MostClient
from .search_types import SearchParams
from .types import Audio, Text


class MostSearcher(object):
    def __init__(self, client: MostClient,
                 data_source: Literal["text", "audio"]):
        self.client = client
        self.data_source = data_source

    def count(self,
              filter: Optional[SearchParams] = None):
        if filter is None:
            filter = SearchParams()

        resp = self.client.get(f"/{self.client.client_id}/{self.data_source}/count",
                               params={
                                   "filter": filter.to_json(),
                               })
        if resp.status_code >= 400:
            raise RuntimeError("Audio can't be indexed")
        return resp.json()

    def distinct(self,
                 key: str,
                 filter: Optional[SearchParams] = None) -> List[str]:
        """
        Distinct values of key.
        :param key: key should be stored in info (fetch_info, store_info)
        :return:
        """
        if filter is None:
            filter = SearchParams()
        resp = self.client.get(f"/{self.client.client_id}/{self.data_source}/distinct",
                               params={"filter": filter.to_json(),
                                       "key": key})
        if resp.status_code >= 400:
            raise RuntimeError("Key is not valid")
        return resp.json()

    def search(self,
               filter: Optional[SearchParams] = None,
               limit: int = 10) -> List[Audio | Text]:
        if filter is None:
            filter = SearchParams()
        resp = self.client.get(f"/{self.client.client_id}/{self.data_source}/search",
                                params={
                                    "filter": filter.to_json(),
                                    "limit": limit,
                                })
        if resp.status_code >= 400:
            raise RuntimeError("Audio can't be indexed")
        audio_list = resp.json()
        return self.client.retort.load(audio_list, List[Audio])



from .api import MostClient
from .async_api import AsyncMostClient
from .trainer_api import Trainer
from .async_trainer_api import AsyncTrainer
from .searcher import MostSearcher
from .async_searcher import AsyncMostClient
from .search_types import SearchParams, IDCondition, ChannelsCondition, DurationCondition, ResultsCondition, StoredInfoCondition


from dataclasses import dataclass, field
from typing import List, Optional
from dataclasses_json import DataClassJsonMixin, dataclass_json


@dataclass_json
@dataclass
class IDCondition(DataClassJsonMixin):
    equal: Optional[str] = None
    in_set: Optional[List[str]] = None
    greater_than: Optional[str] = None
    less_than: Optional[str] = None


@dataclass_json
@dataclass
class ChannelsCondition(DataClassJsonMixin):
    equal: Optional[int] = None


@dataclass_json
@dataclass
class DurationCondition(DataClassJsonMixin):
    greater_than: Optional[int] = None
    less_than: Optional[int] = None


@dataclass_json
@dataclass
class StoredInfoCondition(DataClassJsonMixin):
    key: str
    match: Optional[int | str | float] = None
    starts_with: Optional[str] = None
    ends_with: Optional[str] = None
    greater_than: Optional[int | str | float] = None
    less_than: Optional[int | str | float] = None


@dataclass_json
@dataclass
class ResultsCondition(DataClassJsonMixin):
    column_idx: int
    subcolumn_idx: int
    model_id: str
    score_equal: Optional[int] = None
    score_in_set: Optional[List[int]] = None
    score_greater_than: Optional[int] = None
    score_less_than: Optional[int] = None

    def create_from(self, client,
                    column: str, subcolumn: str,
                    score_equal: Optional[int] = None,
                    score_in_set: Optional[List[int]] = None,
                    score_greater_than: Optional[int] = None,
                    score_less_than: Optional[int] = None,
                    modified_scores: bool = False) -> 'ResultsCondition':
        from .api import MostClient
        client: MostClient
        script = client.get_model_script()
        column_idx = [column.name for column in script.columns].index(column)
        subcolumn_idx = script.columns[column_idx].subcolumns.index(subcolumn)

        if modified_scores:
            score_modifier = client.get_score_modifier()
            if score_equal is not None:
                score_equal = score_modifier.unmodify_single(column, subcolumn,
                                                             score_equal,
                                                             bound="strict")
            if score_in_set is not None:
                score_in_set = [score_modifier.unmodify_single(column, subcolumn,
                                                               score,
                                                               bound="strict")
                                for score in score_in_set]
            if score_greater_than is not None:
                score_greater_than = score_modifier.unmodify_single(column, subcolumn,
                                                                    score_greater_than,
                                                                    bound="upper")

            if score_less_than is not None:
                score_less_than = score_modifier.unmodify_single(column, subcolumn,
                                                                 score_less_than,
                                                                 bound="lower")

        return ResultsCondition(model_id=client.model_id,
                                column_idx=column_idx,
                                subcolumn_idx=subcolumn_idx,
                                score_equal=score_equal,
                                score_in_set=score_in_set,
                                score_greater_than=score_greater_than,
                                score_less_than=score_less_than)

    async def acreate_from(self, client,
                           column: str, subcolumn: str,
                           score_equal: Optional[int] = None,
                           score_in_set: Optional[List[int]] = None,
                           score_greater_than: Optional[int] = None,
                           score_less_than: Optional[int] = None,
                           modified_scores: bool = False) -> 'ResultsCondition':
        from .async_api import AsyncMostClient
        client: AsyncMostClient
        script = await client.get_model_script()
        column_idx = [column.name for column in script.columns].index(column)
        subcolumn_idx = script.columns[column_idx].subcolumns.index(subcolumn)

        if modified_scores:
            score_modifier = await client.get_score_modifier()
            if score_equal is not None:
                score_equal = score_modifier.unmodify_single(column, subcolumn,
                                                             score_equal,
                                                             bound="strict")
            if score_in_set is not None:
                score_in_set = [score_modifier.unmodify_single(column, subcolumn,
                                                               score,
                                                               bound="strict")
                                for score in score_in_set]
            if score_greater_than is not None:
                score_greater_than = score_modifier.unmodify_single(column, subcolumn,
                                                                    score_greater_than,
                                                                    bound="upper")

            if score_less_than is not None:
                score_less_than = score_modifier.unmodify_single(column, subcolumn,
                                                                 score_less_than,
                                                                 bound="lower")

        return ResultsCondition(model_id=client.model_id,
                                column_idx=column_idx,
                                subcolumn_idx=subcolumn_idx,
                                score_equal=score_equal,
                                score_in_set=score_in_set,
                                score_greater_than=score_greater_than,
                                score_less_than=score_less_than)


@dataclass_json
@dataclass
class SearchParams(DataClassJsonMixin):
    must: List[StoredInfoCondition | ResultsCondition | DurationCondition | ChannelsCondition | IDCondition ] = field(default_factory=list)
    should: List[StoredInfoCondition | ResultsCondition | DurationCondition | ChannelsCondition | IDCondition ] = field(default_factory=list)
    must_not: List[StoredInfoCondition | ResultsCondition | DurationCondition | ChannelsCondition | IDCondition ] = field(default_factory=list)
    should_not: List[StoredInfoCondition | ResultsCondition | DurationCondition | ChannelsCondition | IDCondition ] = field(default_factory=list)



from typing import List

from . import AsyncMostClient
from .types import HumanFeedback


class AsyncTrainer(object):
    def __init__(self, client: AsyncMostClient):
        super(AsyncTrainer, self).__init__()
        self.client = client
        if self.client.model_id is None:
            raise RuntimeError("Train must be implemented for stable model_id")

    async def fit(self, data: List[HumanFeedback]):
        resp = await self.client.put(f"/{self.client.client_id}/model/{self.client.model_id}/data",
                                     json={"data": [hf.to_dict() for hf in data]})
        return self

    async def evaluate(self, data: List[HumanFeedback]):
        gt_data = await self.get_data_points()
        return HumanFeedback.calculate_accuracy(data, gt_data)

    async def get_data_points(self) -> List[HumanFeedback]:
        resp = await self.client.get(f"/{self.client.client_id}/model/{self.client.model_id}/data")
        audio_list = resp.json()
        return self.client.retort.load(audio_list, List[HumanFeedback])



from setuptools import setup
from setuptools import find_packages


with open('requirements.txt', 'r') as f:
    requirements = f.read()


setup(
    name='most-client',
    version='1.0.39',
    python_requires=f'>=3.6',
    description='Most AI API for https://the-most.ai',
    url='https://github.com/the-most-ai/most-client',
    author='George Kasparyants',
    author_email='george@the-most.ai',
    license='',
    packages=find_packages(include=['most', 'most.*']),
    install_requires=requirements,
    zip_safe=True,
    include_package_data=True,
    exclude_package_data={'': ['notebooks']},
    classifiers=[
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6"
    ],
)
